@@ -4,6 +4,7 @@ ffibuild = FFI()
 ffibuild.set_source(
     "_notcurses",
     """
+    #include <notcurses/direct.h>
     #include <notcurses/notcurses.h>
     """,
     libraries=["notcurses"],
@@ -82,9 +83,11 @@ typedef struct notcurses_options {
   uint64_t flags;
 } notcurses_options;
 struct notcurses* notcurses_init(const notcurses_options*, FILE*);
+void notcurses_version_components(int* major, int* minor, int* patch, int* tweak);
 int notcurses_lex_margins(const char* op, notcurses_options* opts);
 int notcurses_stop(struct notcurses*);
 int notcurses_render(struct notcurses*);
+int notcurses_render_to_file(struct notcurses* nc, FILE* fp);
 struct ncplane* notcurses_stdplane(struct notcurses*);
 const struct ncplane* notcurses_stdplane_const(const struct notcurses* nc);
 int ncplane_set_base_cell(struct ncplane* ncp, const cell* c);
@@ -206,6 +209,7 @@ int ncplane_putwc_yx(struct ncplane* n, int y, int x, wchar_t w);
 int ncplane_putwc(struct ncplane* n, wchar_t w);
 int ncplane_putegc_yx(struct ncplane* n, int y, int x, const char* gclust, int* sbytes);
 int ncplane_putstr_aligned(struct ncplane* n, int y, ncalign_e align, const char* s);
+int ncplane_putstr_stainable(struct ncplane* n, const char* s);
 struct ncplane* ncplane_dup(const struct ncplane* n, void* opaque);
 void cell_init(cell* c);
 int cell_load(struct ncplane* n, cell* c, const char* gcluster);
@@ -384,11 +388,6 @@ bool ncmenu_offer_input(struct ncmenu* n, const struct ncinput* nc);
 int ncmenu_destroy(struct ncmenu* n);
 const char* ncmetric(uintmax_t val, unsigned decimal, char* buf, int omitdec, unsigned mult, int uprefix);
 typedef struct ncreel_options {
-  int min_supported_cols;
-  int min_supported_rows;
-  int max_supported_cols;
-  int max_supported_rows;
-  int toff, roff, boff, loff;
   unsigned bordermask;
   uint64_t borderchan;
   unsigned tabletmask;
@@ -397,15 +396,12 @@ typedef struct ncreel_options {
   uint64_t bgchannel;
   unsigned flags;      // bitfield over NCREEL_OPTION_*
 } ncreel_options;
-struct ncreel* ncreel_create(struct ncplane* nc, const ncreel_options* popts, int efd);
+struct ncreel* ncreel_create(struct ncplane* nc, const ncreel_options* popts);
 struct ncplane* ncreel_plane(struct ncreel* pr);
 typedef int (*tabletcb)(struct nctablet* t, int begx, int begy, int maxx, int maxy, bool cliptop);
 struct nctablet* ncreel_add(struct ncreel* pr, struct nctablet* after, struct nctablet* before, tabletcb cb, void* opaque);
 int ncreel_tabletcount(const struct ncreel* pr);
-int ncreel_touch(struct ncreel* pr, struct nctablet* t);
 int ncreel_del(struct ncreel* pr, struct nctablet* t);
-int ncreel_del_focused(struct ncreel* pr);
-int ncreel_move(struct ncreel* pr, int x, int y);
 int ncreel_redraw(struct ncreel* pr);
 struct nctablet* ncreel_focused(struct ncreel* pr);
 struct nctablet* ncreel_next(struct ncreel* pr);
@@ -489,7 +485,7 @@ struct ncdirect* ncdirect_init(const char* termtype, FILE* fp);
 int ncdirect_bg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
 int ncdirect_fg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
 int ncdirect_palette_size(const struct ncdirect* nc);
-int ncdirect_putc(struct ncdirect* nc, uint64_t channels, const char* egc);
+int ncdirect_putstr(struct ncdirect* nc, uint64_t channels, const char* utf8);
 int ncdirect_fg(struct ncdirect* n, unsigned rgb);
 int ncdirect_bg(struct ncdirect* n, unsigned rgb);
 int ncdirect_styles_set(struct ncdirect* n, unsigned stylebits);
@@ -506,6 +502,13 @@ int ncdirect_cursor_up(struct ncdirect* nc, int num);
 int ncdirect_cursor_left(struct ncdirect* nc, int num);
 int ncdirect_cursor_right(struct ncdirect* nc, int num);
 int ncdirect_cursor_down(struct ncdirect* nc, int num);
+int ncdirect_hline_interp(struct ncdirect* n, const char* egc, int len, uint64_t h1, uint64_t h2);
+int ncdirect_vline_interp(struct ncdirect* n, const char* egc, int len, uint64_t h1, uint64_t h2);
+int ncdirect_box(struct ncdirect* n, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, const wchar_t* wchars, int ylen, int xlen, unsigned ctlword);
+int ncdirect_rounded_box(struct ncdirect* n, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ylen, int xlen, unsigned ctlword);
+int ncdirect_double_box(struct ncdirect* n, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ylen, int xlen, unsigned ctlword);
+bool ncdirect_canopen_images(const struct ncdirect* n);
+bool ncdirect_canutf8(const struct ncdirect* n);
 nc_err_e ncdirect_render_image(struct ncdirect* n, const char* filename, ncalign_e align, ncblitter_e blitter, ncscale_e scale);
 """)
 
